@@ -24,6 +24,8 @@
  * Each #GdictContext has a list of databases, that is dictionaries that
  * can be queried. #GdictDatabaseChooser is a widget that queries a given
  * #GdictContext and displays the list of available databases.
+ *
+ * #GdictDatabaseChooser is available since Gdict 0.10
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,10 +40,11 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkbindings.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include "gdict-database-chooser.h"
 #include "gdict-utils.h"
+#include "gdict-debug.h"
 #include "gdict-private.h"
 #include "gdict-enum-types.h"
 #include "gdict-marshal.h"
@@ -84,6 +87,7 @@ enum
   DB_COLUMN_TYPE,
   DB_COLUMN_NAME,
   DB_COLUMN_DESCRIPTION,
+  DB_COLUMN_CURRENT,
 
   DB_N_COLUMNS
 };
@@ -99,7 +103,7 @@ enum
 enum
 {
   DATABASE_ACTIVATED,
-  CLOSED,
+  SELECTION_CHANGED,
 
   LAST_SIGNAL
 };
@@ -124,7 +128,7 @@ set_gdict_context (GdictDatabaseChooser *chooser,
     {
       if (priv->start_id)
         {
-          _gdict_debug ("Removing old context handlers\n");
+          GDICT_NOTE (CHOOSER, "Removing old context handlers");
           
           g_signal_handler_disconnect (priv->context, priv->start_id);
           g_signal_handler_disconnect (priv->context, priv->match_id);
@@ -142,9 +146,12 @@ set_gdict_context (GdictDatabaseChooser *chooser,
           priv->error_id = 0;
         }
 
-      _gdict_debug ("Removing old context\n");
+      GDICT_NOTE (CHOOSER, "Removing old context");
       
       g_object_unref (G_OBJECT (priv->context));
+
+      priv->context = NULL;
+      priv->results = -1;
     }
 
   if (!context)
@@ -157,10 +164,10 @@ set_gdict_context (GdictDatabaseChooser *chooser,
       return;
     }
 
-  _gdict_debug ("Setting new context\n");
+  GDICT_NOTE (CHOOSER, "Setting new context");
     
-  priv->context = context;
-  g_object_ref (G_OBJECT (priv->context));
+  priv->context = g_object_ref (context);
+  priv->results = 0;
 }
 
 static void
@@ -169,19 +176,34 @@ gdict_database_chooser_finalize (GObject *gobject)
   GdictDatabaseChooser *chooser = GDICT_DATABASE_CHOOSER (gobject);
   GdictDatabaseChooserPrivate *priv = chooser->priv;
 
-  if (priv->context)
-    set_gdict_context (chooser, NULL);
-
-  if (priv->busy_cursor)
-    gdk_cursor_unref (priv->busy_cursor);
-
-  g_object_unref (priv->store);
-
   g_free (priv->current_db);
   
   G_OBJECT_CLASS (gdict_database_chooser_parent_class)->finalize (gobject);
 }
 
+static void
+gdict_database_chooser_dispose (GObject *gobject)
+{
+  GdictDatabaseChooser *chooser = GDICT_DATABASE_CHOOSER (gobject);
+  GdictDatabaseChooserPrivate *priv = chooser->priv;
+
+  set_gdict_context (chooser, NULL);
+
+  if (priv->busy_cursor)
+    {
+      gdk_cursor_unref (priv->busy_cursor);
+      priv->busy_cursor = NULL;
+    }
+
+  if (priv->store)
+    {
+      g_object_unref (priv->store);
+      priv->store = NULL;
+    }
+
+  G_OBJECT_CLASS (gdict_database_chooser_parent_class)->dispose (gobject);
+}
+
 static void
 gdict_database_chooser_set_property (GObject      *gobject,
 				     guint         prop_id,
@@ -286,11 +308,19 @@ clear_button_clicked_cb (GtkWidget *widget,
   gdict_database_chooser_clear (chooser);
 }
 
+static void
+selection_changed_cb (GtkTreeSelection *selection,
+                      gpointer          user_data)
+{
+  g_signal_emit (user_data, db_chooser_signals[SELECTION_CHANGED], 0);
+}
+
 static GObject *
 gdict_database_chooser_constructor (GType                  type,
 				    guint                  n_params,
 				    GObjectConstructParam *params)
 {
+  GObjectClass *parent_class;
   GObject *object;
   GdictDatabaseChooser *chooser;
   GdictDatabaseChooserPrivate *priv;
@@ -299,9 +329,8 @@ gdict_database_chooser_constructor (GType                  type,
   GtkTreeViewColumn *column;
   GtkWidget *hbox;
 
-  object = G_OBJECT_CLASS (gdict_database_chooser_parent_class)->constructor (type,
-		  							      n_params,
-									      params);
+  parent_class = G_OBJECT_CLASS (gdict_database_chooser_parent_class);
+  object = parent_class->constructor (type, n_params, params);
 
   chooser = GDICT_DATABASE_CHOOSER (object);
   priv = chooser->priv;
@@ -322,6 +351,7 @@ gdict_database_chooser_constructor (GType                  type,
   column = gtk_tree_view_column_new_with_attributes ("databases",
 		  				     renderer,
 						     "text", DB_COLUMN_DESCRIPTION,
+                                                     "weight", DB_COLUMN_CURRENT,
 						     NULL);
   priv->treeview = gtk_tree_view_new ();
   gtk_widget_set_composite_name (priv->treeview, "gdict-database-chooser-treeview");
@@ -329,6 +359,9 @@ gdict_database_chooser_constructor (GType                  type,
 		  	   GTK_TREE_MODEL (priv->store));
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (priv->treeview), FALSE);
   gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
+  g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview)),
+                    "changed", G_CALLBACK (selection_changed_cb),
+                    chooser);
   g_signal_connect (priv->treeview, "row-activated",
 		    G_CALLBACK (row_activated_cb), chooser);
   gtk_container_add (GTK_CONTAINER (sw), priv->treeview);
@@ -375,18 +408,52 @@ gdict_database_chooser_class_init (GdictDatabaseChooserClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   
   gobject_class->finalize = gdict_database_chooser_finalize;
+  gobject_class->dispose = gdict_database_chooser_dispose;
   gobject_class->set_property = gdict_database_chooser_set_property;
   gobject_class->get_property = gdict_database_chooser_get_property;
   gobject_class->constructor = gdict_database_chooser_constructor;
-  
+
+  /**
+   * GdictDatabaseChooser:context:
+   *
+   * The #GdictContext used to retrieve the list of available databases.
+   *
+   * Since: 0.10
+   */
   g_object_class_install_property (gobject_class,
   				   PROP_CONTEXT,
   				   g_param_spec_object ("context",
   				   			"Context",
   				   			"The GdictContext object used to get the list of databases",
   				   			GDICT_TYPE_CONTEXT,
-  				   			(G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT)));
+  				   			(G_PARAM_READWRITE | G_PARAM_CONSTRUCT)));
+  /**
+   * GdictDatabaseChooser:count:
+   *
+   * The number of displayed databases or, if no #GdictContext is set, -1.
+   *
+   * Since: 0.12
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_COUNT,
+                                   g_param_spec_int ("count",
+                                                     "Count",
+                                                     "The number of available databases",
+                                                     -1, G_MAXINT, -1,
+                                                     G_PARAM_READABLE));
 
+  /**
+   * GdictDatabaseChooser::database-activated:
+   * @chooser: the database chooser that received the signal
+   * @name: the name of the activated database
+   * @description: the description of the activated database
+   *
+   * The ::database-activated signal is emitted each time the user
+   * activated a row in the database chooser widget, either by double
+   * clicking on it or by a keyboard event.
+   *
+   * Since: 0.10
+   */
   db_chooser_signals[DATABASE_ACTIVATED] =
     g_signal_new ("database-activated",
 		  G_OBJECT_CLASS_TYPE (gobject_class),
@@ -397,7 +464,24 @@ gdict_database_chooser_class_init (GdictDatabaseChooserClass *klass)
 		  G_TYPE_NONE, 2,
 		  G_TYPE_STRING,
 		  G_TYPE_STRING);
-  
+  /**
+   * GdictDatabaseChooser::selection-changed:
+   * @chooser: the database chooser that received the signal
+   *
+   * The ::selection-changed signal is emitted each time the selection
+   * inside the database chooser has been changed.
+   *
+   * Since: 0.12
+   */
+  db_chooser_signals[SELECTION_CHANGED] =
+    g_signal_new ("selection-changed",
+                  G_OBJECT_CLASS_TYPE (gobject_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GdictDatabaseChooserClass, selection_changed),
+                  NULL, NULL,
+                  gdict_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
   g_type_class_add_private (gobject_class, sizeof (GdictDatabaseChooserPrivate));
 }
 
@@ -412,9 +496,10 @@ gdict_database_chooser_init (GdictDatabaseChooser *chooser)
   priv->context = NULL;
 
   priv->store = gtk_list_store_new (DB_N_COLUMNS,
-		                    G_TYPE_INT,    /* DBType */
+		                    G_TYPE_INT,    /* db_type */
 		                    G_TYPE_STRING, /* db_name */
-				    G_TYPE_STRING  /* db_desc */);
+				    G_TYPE_STRING, /* db_desc */
+                                    G_TYPE_INT     /* db_current */);
 
   priv->start_id = 0;
   priv->end_id = 0;
@@ -425,11 +510,14 @@ gdict_database_chooser_init (GdictDatabaseChooser *chooser)
 /**
  * gdict_database_chooser_new:
  *
- * FIXME
+ * Creates a new #GdictDatabaseChooser widget. A Database chooser widget
+ * can be used to display the list of available databases on a dictionary
+ * source using the #GdictContext representing it. After creation, the
+ * #GdictContext can be set using gdict_database_chooser_set_context().
  *
- * Return value: FIXME
+ * Return value: the newly created #GdictDatabaseChooser widget.
  *
- * Since: 0.9
+ * Since: 0.10
  */
 GtkWidget *
 gdict_database_chooser_new (void)
@@ -441,11 +529,12 @@ gdict_database_chooser_new (void)
  * gdict_database_chooser_new_with_context:
  * @context: a #GdictContext
  *
- * FIXME
+ * Creates a new #GdictDatabaseChooser, using @context as the representation
+ * of the dictionary source to query for the list of available databases.
  *
- * Return value: FIXME
+ * Return value: the newly created #GdictDatabaseChooser widget.
  *
- * Since: 0.9
+ * Since: 0.10
  */
 GtkWidget *
 gdict_database_chooser_new_with_context (GdictContext *context)
@@ -461,11 +550,11 @@ gdict_database_chooser_new_with_context (GdictContext *context)
  * gdict_database_chooser_get_context:
  * @chooser: a #GdictDatabaseChooser
  *
- * FIXME
+ * Retrieves the #GdictContext used by @chooser.
  *
- * Return value: a #GdictContext
+ * Return value: a #GdictContext or %NULL
  *
- * Since:
+ * Since: 0.10
  */
 GdictContext *
 gdict_database_chooser_get_context (GdictDatabaseChooser *chooser)
@@ -480,9 +569,10 @@ gdict_database_chooser_get_context (GdictDatabaseChooser *chooser)
  * @chooser: a #GdictDatabaseChooser
  * @context: a #GdictContext
  *
- * FIXME
+ * Sets the #GdictContext to be used to query a dictionary source
+ * for the list of available databases.
  *
- * Since:
+ * Since: 0.10
  */
 void
 gdict_database_chooser_set_context (GdictDatabaseChooser *chooser,
@@ -499,53 +589,114 @@ gdict_database_chooser_set_context (GdictDatabaseChooser *chooser,
 /**
  * gdict_database_chooser_get_databases:
  * @chooser: a #GdictDatabaseChooser
- * @length: FIXME
+ * @length: return location for the length of the returned vector
  *
- * FIXME
+ * Gets the list of available database names.
  *
- * Return value: FIXME
+ * Return value: a newly allocated, %NULL terminated string vector
+ *   containing database names. Use g_strfreev() to deallocate it.
  *
- * Since:
+ * Since: 0.10
  */
 gchar **
 gdict_database_chooser_get_databases (GdictDatabaseChooser  *chooser,
-				      gsize                  length)
+				      gsize                 *length)
 {
+  GdictDatabaseChooserPrivate *priv;
+  GtkTreeIter iter;
+  gchar **retval;
+  gsize i;
+
   g_return_val_if_fail (GDICT_IS_DATABASE_CHOOSER (chooser), NULL);
 
-  return NULL;
+  priv = chooser->priv;
+
+  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
+    return NULL;
+
+  i = 0;
+  retval = g_new (gchar*, priv->results);
+
+  do
+    {
+      gchar *db_name;
+
+      gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
+                          DB_COLUMN_NAME, &db_name,
+                          -1);
+
+      retval[i++] = db_name;
+    }
+  while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store), &iter));
+
+  retval[i] = NULL;
+
+  if (length)
+    *length = i;
+
+  return retval;
 }
 
 /**
  * gdict_database_chooser_has_database:
  * @chooser: a #GdictDatabaseChooser
- * @database: FIXME
+ * @database: the name of a database
  *
- * FIXME
+ * Checks whether the @chooser displays @database
  *
- * Return value: FIXME
+ * Return value: %TRUE if the search database name is present
  *
- * Since:
+ * Since: 0.10
  */
 gboolean
 gdict_database_chooser_has_database (GdictDatabaseChooser *chooser,
 				     const gchar          *database)
 {
+  GdictDatabaseChooserPrivate *priv;
+  GtkTreeIter iter;
+  gboolean retval;
+
   g_return_val_if_fail (GDICT_IS_DATABASE_CHOOSER (chooser), FALSE);
   g_return_val_if_fail (database != NULL, FALSE);
 
-  return FALSE;
+  priv = chooser->priv;
+
+  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->store), &iter))
+    return FALSE;
+
+  retval = FALSE;
+
+  do
+    {
+      gchar *db_name;
+
+      gtk_tree_model_get (GTK_TREE_MODEL (priv->store), &iter,
+                          DB_COLUMN_NAME, &db_name,
+                          -1);
+      
+      if (strcmp (db_name, database) == 0)
+        {
+          g_free (db_name);
+          retval = TRUE;
+          break;
+        }
+      
+      g_free (db_name);
+    }
+  while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->store), &iter));
+
+  return retval;
 }
 
 /**
- * gdict_database_chooser_count_dayabases:
+ * gdict_database_chooser_count_databases:
  * @chooser: a #GdictDatabaseChooser
  *
  * Returns the number of databases found.
  *
- * Return value: the number of databases or -1 if case of error
+ * Return value: the number of databases or -1 if no context is set
  *
- * Since:
+ * Since: 0.10
  */
 gint
 gdict_database_chooser_count_databases (GdictDatabaseChooser *chooser)
@@ -592,22 +743,28 @@ database_found_cb (GdictContext  *context,
   GdictDatabaseChooser *chooser = GDICT_DATABASE_CHOOSER (user_data);
   GdictDatabaseChooserPrivate *priv = chooser->priv;
   GtkTreeIter iter;
+  const gchar *name, *full_name;
+  gint weight = PANGO_WEIGHT_NORMAL;
 
-  _gdict_debug ("DATABASE: `%s' (`%s')\n",
-		gdict_database_get_name (database),
-		gdict_database_get_full_name (database));
+  name = gdict_database_get_name (database);
+  full_name = gdict_database_get_full_name (database);
 
+  if (priv->current_db && !strcmp (priv->current_db, name))
+    weight = PANGO_WEIGHT_BOLD;
+
+  GDICT_NOTE (CHOOSER, "DATABASE: `%s' (`%s')",
+              name,
+              full_name);
+  
   gtk_list_store_append (priv->store, &iter);
   gtk_list_store_set (priv->store, &iter,
 		      DB_COLUMN_TYPE, DATABASE_NAME,
-		      DB_COLUMN_NAME, gdict_database_get_name (database),
-		      DB_COLUMN_DESCRIPTION, gdict_database_get_full_name (database),
+		      DB_COLUMN_NAME, name,
+		      DB_COLUMN_DESCRIPTION, full_name,
+                      DB_COLUMN_CURRENT, weight,
 		      -1);
 
-  if (priv->results == -1)
-    priv->results = 1;
-  else
-    priv->results += 1;
+  priv->results += 1;
 }
 
 static void
@@ -621,6 +778,7 @@ error_cb (GdictContext *context,
     gdk_window_set_cursor (GTK_WIDGET (chooser)->window, NULL);
 
   chooser->priv->is_searching = FALSE;
+  chooser->priv->results = 0;
 }
 
 /**
@@ -629,7 +787,7 @@ error_cb (GdictContext *context,
  *
  * Reloads the list of available databases.
  *
- * Since:
+ * Since: 0.10
  */
 void
 gdict_database_chooser_refresh (GdictDatabaseChooser *chooser)
@@ -650,12 +808,7 @@ gdict_database_chooser_refresh (GdictDatabaseChooser *chooser)
     }
 
   if (priv->is_searching)
-    {
-      _gdict_show_error_dialog (NULL,
-				_("Another search is in progress"),
-				_("Please wait until the current search ends."));
-      return;
-    }
+    return;
 
   gdict_database_chooser_clear (chooser);
 
@@ -690,7 +843,9 @@ gdict_database_chooser_refresh (GdictDatabaseChooser *chooser)
 			  DB_COLUMN_DESCRIPTION, NULL,
 			  -1);
 
-      _gdict_debug ("Error while searching: %s", db_error->message);
+      g_warning ("Error while looking for databases: %s",
+                 db_error->message);
+
       g_error_free (db_error);
     }
 }
@@ -701,7 +856,7 @@ gdict_database_chooser_refresh (GdictDatabaseChooser *chooser)
  *
  * Clears @chooser.
  *
- * Since:
+ * Since: 0.10
  */
 void
 gdict_database_chooser_clear (GdictDatabaseChooser *chooser)
@@ -715,7 +870,7 @@ gdict_database_chooser_clear (GdictDatabaseChooser *chooser)
   gtk_tree_view_set_model (GTK_TREE_VIEW (priv->treeview), NULL);
 
   gtk_list_store_clear (priv->store);
-  priv->results = -1;
+  priv->results = 0;
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (priv->treeview),
 		  	   GTK_TREE_MODEL (priv->store));
@@ -743,9 +898,6 @@ scan_for_db_name (GtkTreeModel *model,
   if (!select_data)
     return TRUE;
 
-  if (select_data->found)
-    return TRUE;
-
   gtk_tree_model_get (model, iter, DB_COLUMN_NAME, &db_name, -1);
   if (!db_name)
     return FALSE;
@@ -759,8 +911,16 @@ scan_for_db_name (GtkTreeModel *model,
 
       tree_view = GTK_TREE_VIEW (select_data->chooser->priv->treeview);
       if (select_data->do_activate)
-        gtk_tree_view_row_activated (tree_view, path,
-                                     gtk_tree_view_get_column (tree_view, 2));
+        {
+          GtkTreeViewColumn *column;
+
+          gtk_list_store_set (GTK_LIST_STORE (model), iter,
+                              DB_COLUMN_CURRENT, PANGO_WEIGHT_BOLD,
+                              -1);
+
+          column = gtk_tree_view_get_column (tree_view, 0);
+          gtk_tree_view_row_activated (tree_view, path, column);
+        }
 
       selection = gtk_tree_view_get_selection (tree_view);
       if (select_data->do_select)
@@ -768,18 +928,35 @@ scan_for_db_name (GtkTreeModel *model,
       else
         gtk_tree_selection_unselect_path (selection, path);
     }
+  else
+    {
+      gtk_list_store_set (GTK_LIST_STORE (model), iter,
+                          DB_COLUMN_CURRENT, PANGO_WEIGHT_NORMAL,
+                          -1);
+    }
 
   g_free (db_name);
 
-  return select_data->found;
+  return FALSE;
 }
 
+/**
+ * gdict_database_chooser_select_database:
+ * @chooser: a #GdictDatabaseChooser
+ * @db_name: name of the database to select
+ *
+ * Selects the database with @db_name inside the @chooser widget.
+ *
+ * Return value: %TRUE if the database was found and selected
+ *
+ * Since: 0.10
+ */
 gboolean
 gdict_database_chooser_select_database (GdictDatabaseChooser *chooser,
                                         const gchar          *db_name)
 {
   GdictDatabaseChooserPrivate *priv;
-  SelectData *data;
+  SelectData data;
   gboolean retval;
 
   g_return_val_if_fail (GDICT_IS_DATABASE_CHOOSER (chooser), FALSE);
@@ -787,31 +964,40 @@ gdict_database_chooser_select_database (GdictDatabaseChooser *chooser,
 
   priv = chooser->priv;
 
-  data = g_slice_new0 (SelectData);
-  data->db_name = g_strdup (db_name);
-  data->chooser = chooser;
-  data->found = FALSE;
-  data->do_select = TRUE;
-  data->do_activate = FALSE;
+  data.db_name = g_strdup (db_name);
+  data.chooser = chooser;
+  data.found = FALSE;
+  data.do_select = TRUE;
+  data.do_activate = FALSE;
 
   gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
                           scan_for_db_name,
-                          data);
+                          &data);
 
-  retval = data->found;
+  retval = data.found;
 
-  g_free (data->db_name);
-  g_slice_free (SelectData, data);
+  g_free (data.db_name);
 
   return retval;
 }
 
+/**
+ * gdict_database_chooser_unselect_database:
+ * @chooser: a #GdictDatabaseChooser
+ * @db_name: name of the database to unselect
+ *
+ * Unselects the database @db_name inside the @chooser widget
+ *
+ * Return value: %TRUE if the database was found and unselected
+ *
+ * Since: 0.10
+ */
 gboolean
 gdict_database_chooser_unselect_database (GdictDatabaseChooser *chooser,
                                           const gchar          *db_name)
 {
   GdictDatabaseChooserPrivate *priv;
-  SelectData *data;
+  SelectData data;
   gboolean retval;
 
   g_return_val_if_fail (GDICT_IS_DATABASE_CHOOSER (chooser), FALSE);
@@ -819,31 +1005,41 @@ gdict_database_chooser_unselect_database (GdictDatabaseChooser *chooser,
 
   priv = chooser->priv;
 
-  data = g_slice_new0 (SelectData);
-  data->db_name = g_strdup (db_name);
-  data->chooser = chooser;
-  data->found = FALSE;
-  data->do_select = FALSE;
-  data->do_activate = FALSE;
+  data.db_name = g_strdup (db_name);
+  data.chooser = chooser;
+  data.found = FALSE;
+  data.do_select = FALSE;
+  data.do_activate = FALSE;
 
   gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
                           scan_for_db_name,
-                          data);
+                          &data);
 
-  retval = data->found;
+  retval = data.found;
 
-  g_free (data->db_name);
-  g_slice_free (SelectData, data);
+  g_free (data.db_name);
 
   return retval;
 }
 
+/**
+ * gdict_database_chooser_set_current_database:
+ * @chooser: a #GdictDatabaseChooser
+ * @db_name: the name of the database
+ *
+ * Sets @db_name as the current database. This function will select
+ * and activate the corresponding row, if the database is found.
+ *
+ * Return value: %TRUE if the database was found and set
+ *
+ * Since: 0.10
+ */
 gboolean
 gdict_database_chooser_set_current_database (GdictDatabaseChooser *chooser,
                                              const gchar          *db_name)
 {
   GdictDatabaseChooserPrivate *priv;
-  SelectData *data;
+  SelectData data;
   gboolean retval;
 
   g_return_val_if_fail (GDICT_IS_DATABASE_CHOOSER (chooser), FALSE);
@@ -851,25 +1047,40 @@ gdict_database_chooser_set_current_database (GdictDatabaseChooser *chooser,
 
   priv = chooser->priv;
 
-  data = g_slice_new0 (SelectData);
-  data->db_name = g_strdup (db_name);
-  data->chooser = chooser;
-  data->found = FALSE;
-  data->do_select = TRUE;
-  data->do_activate = TRUE;
+  data.db_name = g_strdup (db_name);
+  data.chooser = chooser;
+  data.found = FALSE;
+  data.do_select = TRUE;
+  data.do_activate = TRUE;
 
   gtk_tree_model_foreach (GTK_TREE_MODEL (priv->store),
                           scan_for_db_name,
-                          data);
+                          &data);
 
-  retval = data->found;
+  retval = data.found;
 
-  g_free (data->db_name);
-  g_slice_free (SelectData, data);
+  if (data.found)
+    {
+      g_free (priv->current_db);
+      priv->current_db = data.db_name;
+    }
+  else
+    g_free (data.db_name);
 
   return retval;
 }
 
+/**
+ * gdict_database_chooser_get_current_database:
+ * @chooser: a #GdictDatabaseChooser
+ *
+ * Retrieves the name of the currently selected database inside @chooser
+ *
+ * Return value: the name of the selected database. Use g_free() on the
+ *   returned string when done using it
+ *
+ * Since: 0.10
+ */
 gchar *
 gdict_database_chooser_get_current_database (GdictDatabaseChooser *chooser)
 {
@@ -895,6 +1106,19 @@ gdict_database_chooser_get_current_database (GdictDatabaseChooser *chooser)
   return retval;
 }
 
+/**
+ * gdict_database_chooser_add_button:
+ * @chooser: a #GdictDatabase
+ * @button_text: text of the button
+ *
+ * Adds a #GtkButton with @button_text to the button area on
+ * the bottom of @chooser. The @button_text can also be a
+ * stock ID.
+ *
+ * Return value: the newly packed button.
+ *
+ * Since: 0.10
+ */
 GtkWidget *
 gdict_database_chooser_add_button (GdictDatabaseChooser *chooser,
                                    const gchar          *button_text)
